@@ -2,7 +2,7 @@
 #'
 #' @description
 #' Uniform random sampling for an arbitrary (bounded) [ParamSet].
-#' Constructs 1 uniform sampler per [Param], then passes them to [SamplerHierarchical].
+#' Constructs 1 uniform sampler per parameter, then passes them to [SamplerHierarchical].
 #' Hence, also works for [ParamSet]s sets with dependencies.
 #'
 #' @template param_param_set
@@ -14,9 +14,11 @@ SamplerUnif = R6Class("SamplerUnif", inherit = SamplerHierarchical,
   public = list(
     #' @description
     #' Creates a new instance of this [R6][R6::R6Class] class.
+    #' @param param_set ([`ParamSet`])\cr
+    #'   The [`ParamSet`] to associated with this `SamplerUnif`.
     initialize = function(param_set) {
       assert_param_set(param_set, must_bounded = TRUE, no_deps = FALSE, no_untyped = TRUE)
-      samplers = lapply(param_set$params, Sampler1DUnif$new)
+      samplers = lapply(param_set$subspaces(), Sampler1DUnif$new)
       super$initialize(param_set, samplers)
     }
   )
